<li id="fastdt-{{ panel.panel_id }}" class="fastDebugPanelButton">
  <input type="checkbox" data-cookie="fastdt{{ panel.panel_id }}" {% if panel.enabled %}checked title="Disable for next and successive requests"{% else %}title="Enable for next and successive requests"{% endif %}>
  {% if panel.has_content and panel.enabled %}
    <a href="#" title="{{ panel.title }}" class="{{ panel.panel_id }}">
  {% else %}
    <div class="fastdt-contentless{% if not panel.enabled %} fastdt-disabled{% endif %}">
  {% endif %}
  {{ panel.nav_title }}
  {% if panel.enabled %}
    {% with subtitle=panel.nav_subtitle %}
      {% if subtitle %}<br><small>{{ subtitle }}</small>{% endif %}
    {% endwith %}
  {% endif %}
  {% if panel.has_content and panel.enabled %}
    </a>
  {% else %}
    </div>
  {% endif %}
</li>
