<!DOCTYPE html>
<html lang="en">
  <head>
    <title>Debug Toolbar Redirects Panel: {{ status_code }}</title>
    <script type="module" src="{{ url_for('debug_toolbar.static', path='js/redirect.js') }}" async></script>
  </head>
  <body>
    <h1>{{ status_line }}</h1>
    <h2>Location: <a id="redirect_to" href="{{ redirect_to }}">{{ redirect_to }}</a></h2>
    <p class="notice">
      The Debug Toolbar has intercepted a redirect to the above URL for debug viewing purposes. You can click the above link to continue with the redirect as normal.
    </p>
  </body>
</html>
